<!--

#################################################
### THIS FILE WAS AUTOGENERATED! DO NOT EDIT! ###
#################################################
# file to edit: index.ipynb
# command to build the docs after a change: nbdev_build_docs

-->

# online_triplet_loss

> PyTorch conversion of the excellent post on the [same topic in Tensorflow](https://omoindrot.github.io/triplet-loss). Simply an implementation of a triple loss with online mining of candidate triplets used in semi-supervised learning.


## Install

`pip install online_triplet_loss`

Then import with:
`from online_triplet_loss.losses import *`

## How to use

In these examples I use a really large margin, since the embedding space is so small. A more realistic margins seems to be between `0.1 and 2.0`
<div class="codecell" markdown="1">
<div class="input_area" markdown="1">

```
from torch import nn
import torch

model = nn.Embedding(10, 10)
```

</div>

</div>
<div class="codecell" markdown="1">
<div class="input_area" markdown="1">

```
#from online_triplet_loss.losses import *
labels = torch.randint(high=10, size=(5,)) # our five labels

embeddings = model(labels)
print('Labels:', labels)
print('Embeddings:', embeddings)
loss = batch_hard_triplet_loss(labels, embeddings, margin=100)
print('Loss:', loss)
loss.backward()
```

</div>
<div class="output_area" markdown="1">

    Labels: tensor([2, 0, 8, 8, 6])
    Embeddings: tensor([[-2.1302, -1.2041, -0.1863,  0.0326, -1.4517, -1.1717,  0.5536,  0.0911,
              0.6340,  0.9799],
            [ 0.3097, -0.9580, -1.3407,  0.9034, -1.6870, -2.3679, -0.1658,  1.6462,
              0.7228,  1.1883],
            [-0.4034, -1.0991,  0.2177, -0.4273, -0.7944,  1.0775,  0.9443,  1.3429,
              0.1356, -0.9966],
            [-0.4034, -1.0991,  0.2177, -0.4273, -0.7944,  1.0775,  0.9443,  1.3429,
              0.1356, -0.9966],
            [ 0.5503, -2.1766,  0.9868,  0.5284,  0.8618,  1.6336, -0.7546, -0.7549,
              0.1552, -0.5286]], grad_fn=<EmbeddingBackward>)
    Loss: tensor(96.3256, grad_fn=<MeanBackward1>)


</div>

</div>
<div class="codecell" markdown="1">
<div class="input_area" markdown="1">

```
#from online_triplet_loss.losses import *
embeddings = model(labels)
print('Labels:', labels)
print('Embeddings:', embeddings)
loss, fraction_pos = batch_all_triplet_loss(labels, embeddings, squared=False, margin=100)
print('Loss:', loss)
loss.backward()
```

</div>
<div class="output_area" markdown="1">

    Labels: tensor([2, 0, 8, 8, 6])
    Embeddings: tensor([[-2.1302, -1.2041, -0.1863,  0.0326, -1.4517, -1.1717,  0.5536,  0.0911,
              0.6340,  0.9799],
            [ 0.3097, -0.9580, -1.3407,  0.9034, -1.6870, -2.3679, -0.1658,  1.6462,
              0.7228,  1.1883],
            [-0.4034, -1.0991,  0.2177, -0.4273, -0.7944,  1.0775,  0.9443,  1.3429,
              0.1356, -0.9966],
            [-0.4034, -1.0991,  0.2177, -0.4273, -0.7944,  1.0775,  0.9443,  1.3429,
              0.1356, -0.9966],
            [ 0.5503, -2.1766,  0.9868,  0.5284,  0.8618,  1.6336, -0.7546, -0.7549,
              0.1552, -0.5286]], grad_fn=<EmbeddingBackward>)
    tensor(95.8399, grad_fn=<DivBackward0>) tensor(1.)
    Loss: tensor(95.8399, grad_fn=<DivBackward0>)


</div>

</div>

## References
* [Triplet Loss and Online Triplet Mining in Tensorflow](https://github.com/omoindrot/tensorflow-triplet-loss)
* [Facenet paper](https://arxiv.org/abs/1503.03832)
* [adambielski's nice implementation](https://github.com/adambielski/siamese-triplet) (unfortunately context switches between CPU / GPU)
